@@ -31,6 +31,7 @@
 #include <sstream>
 #include <string>
 #include <vector>
+#include <cmath>
 
 #include <dirent.h>
 #include <errno.h>
@@ -77,30 +78,46 @@ void pkgAcquire::Initialize()
 }
 									/*}}}*/
 // Acquire::GetLock - lock directory and prepare for action		/*{{{*/
-static bool SetupAPTPartialDirectory(std::string const &grand, std::string const &parent)
+static bool SetupAPTPartialDirectory(std::string const &grand, std::string const &parent, std::string const &postfix, mode_t const mode)
 {
-   std::string const partial = parent + "partial";
-   mode_t const mode = umask(S_IWGRP | S_IWOTH);
-   bool const creation_fail = (CreateAPTDirectoryIfNeeded(grand, partial) == false &&
-	 CreateAPTDirectoryIfNeeded(parent, partial) == false);
-   umask(mode);
-   if (creation_fail == true)
-      return false;
+   if (_config->FindB("Debug::SetupAPTPartialDirectory::AssumeGood", false))
+      return true;
+   std::string const partial = parent + postfix;
+   bool const partialExists = DirectoryExists(partial);
+   if (partialExists == false)
+   {
+      mode_t const old_umask = umask(S_IWGRP | S_IWOTH);
+      bool const creation_fail = (CreateAPTDirectoryIfNeeded(grand, partial) == false &&
+	    CreateAPTDirectoryIfNeeded(parent, partial) == false);
+      umask(old_umask);
+      if (creation_fail == true)
+	 return false;
+   }
 
    std::string const SandboxUser = _config->Find("APT::Sandbox::User");
-   if (getuid() == 0 && SandboxUser.empty() == false && SandboxUser != "root") // if we aren't root, we can't chown, so don't try it
+   if (getuid() == 0)
    {
-      struct passwd const * const pw = getpwnam(SandboxUser.c_str());
-      struct group const * const gr = getgrnam(ROOT_GROUP);
-      if (pw != NULL && gr != NULL)
+      if (SandboxUser.empty() == false && SandboxUser != "root") // if we aren't root, we can't chown, so don't try it
       {
-         // chown the partial dir
-         if(chown(partial.c_str(), pw->pw_uid, gr->gr_gid) != 0)
-            _error->WarningE("SetupAPTPartialDirectory", "chown to %s:root of directory %s failed", SandboxUser.c_str(), partial.c_str());
+	 struct passwd const * const pw = getpwnam(SandboxUser.c_str());
+	 struct group const * const gr = getgrnam(ROOT_GROUP);
+	 if (pw != NULL && gr != NULL)
+	 {
+	    // chown the partial dir
+	    if(chown(partial.c_str(), pw->pw_uid, gr->gr_gid) != 0)
+	       _error->WarningE("SetupAPTPartialDirectory", "chown to %s:root of directory %s failed", SandboxUser.c_str(), partial.c_str());
+	 }
       }
+      if (chmod(partial.c_str(), mode) != 0)
+	 _error->WarningE("SetupAPTPartialDirectory", "chmod 0700 of directory %s failed", partial.c_str());
+
+   }
+   else if (chmod(partial.c_str(), mode) != 0)
+   {
+      // if we haven't created the dir and aren't root, it is kinda expected that chmod doesn't work
+      if (partialExists == false)
+	 _error->WarningE("SetupAPTPartialDirectory", "chmod 0700 of directory %s failed", partial.c_str());
    }
-   if (chmod(partial.c_str(), 0700) != 0)
-      _error->WarningE("SetupAPTPartialDirectory", "chmod 0700 of directory %s failed", partial.c_str());
 
    _error->PushToStack();
    // remove 'old' FAILED files to stop us from collecting them for no reason
@@ -116,11 +133,15 @@ bool pkgAcquire::Setup(pkgAcquireStatus *Progress, string const &Lock)
    if (Lock.empty())
    {
       string const listDir = _config->FindDir("Dir::State::lists");
-      if (SetupAPTPartialDirectory(_config->FindDir("Dir::State"), listDir) == false)
-	 return _error->Errno("Acquire", _("List directory %spartial is missing."), listDir.c_str());
+      if (SetupAPTPartialDirectory(_config->FindDir("Dir::State"), listDir, "partial", 0700) == false)
+	 return _error->Errno("Acquire", _("List directory %s is missing."), (listDir + "partial").c_str());
+      if (SetupAPTPartialDirectory(_config->FindDir("Dir::State"), listDir, "auxfiles", 0755) == false)
+      {
+	 // not being able to create lists/auxfiles isn't critical as we will use a tmpdir then
+      }
       string const archivesDir = _config->FindDir("Dir::Cache::Archives");
-      if (SetupAPTPartialDirectory(_config->FindDir("Dir::Cache"), archivesDir) == false)
-	 return _error->Errno("Acquire", _("Archives directory %spartial is missing."), archivesDir.c_str());
+      if (SetupAPTPartialDirectory(_config->FindDir("Dir::Cache"), archivesDir, "partial", 0700) == false)
+	 return _error->Errno("Acquire", _("Archives directory %s is missing."), (archivesDir + "partial").c_str());
       return true;
    }
    return GetLock(Lock);
@@ -136,13 +157,20 @@ bool pkgAcquire::GetLock(std::string const &Lock)
 
    if (Lock == listDir)
    {
-      if (SetupAPTPartialDirectory(_config->FindDir("Dir::State"), listDir) == false)
-	 return _error->Errno("Acquire", _("List directory %spartial is missing."), listDir.c_str());
+      if (SetupAPTPartialDirectory(_config->FindDir("Dir::State"), listDir, "partial", 0700) == false)
+	 return _error->Errno("Acquire", _("List directory %s is missing."), (listDir + "partial").c_str());
    }
    if (Lock == archivesDir)
    {
-      if (SetupAPTPartialDirectory(_config->FindDir("Dir::Cache"), archivesDir) == false)
-	 return _error->Errno("Acquire", _("Archives directory %spartial is missing."), archivesDir.c_str());
+      if (SetupAPTPartialDirectory(_config->FindDir("Dir::Cache"), archivesDir, "partial", 0700) == false)
+	 return _error->Errno("Acquire", _("Archives directory %s is missing."), (archivesDir + "partial").c_str());
+   }
+   if (Lock == listDir || Lock == archivesDir)
+   {
+      if (SetupAPTPartialDirectory(_config->FindDir("Dir::State"), listDir, "auxfiles", 0755) == false)
+      {
+	 // not being able to create lists/auxfiles isn't critical as we will use a tmpdir then
+      }
    }
 
    if (_config->FindB("Debug::NoLocking", false) == true)
@@ -287,7 +315,6 @@ static bool CheckForBadItemAndFailIt(pkgAcquire::Item * const Item,
 	 "\nFilename: " + Item->DestFile +
 	 "\nFailReason: WeakHashSums";
 
-      auto SavedDesc = Item->GetItemDesc();
       Item->Status = pkgAcquire::Item::StatAuthError;
       Item->Failed(Message, Config);
       if (Log != nullptr)
@@ -302,7 +329,10 @@ void pkgAcquire::Enqueue(ItemDesc &Item)
    const MethodConfig *Config;
    string Name = QueueName(Item.URI,Config);
    if (Name.empty() == true)
+   {
+      Item.Owner->Status = pkgAcquire::Item::StatError;
       return;
+   }
 
    /* the check for running avoids that we produce errors
       in logging before we actually have started, which would
@@ -437,7 +467,7 @@ string pkgAcquire::QueueName(string Uri,MethodConfig const *&Config)
       clog << "Found " << Instances << " instances of " << U.Access << endl;
    }
 
-   if (Instances >= (unsigned int)_config->FindI("Acquire::QueueHost::Limit",10))
+   if (Instances >= static_cast<decltype(Instances)>(_config->FindI("Acquire::QueueHost::Limit",10)))
       return U.Access;
 
    return FullQueueName;
@@ -768,11 +798,12 @@ bool pkgAcquire::Clean(string Dir)
    for (struct dirent *E = readdir(D); E != nullptr; E = readdir(D))
    {
       // Skip some entries
-      if (strcmp(E->d_name,"lock") == 0 ||
-	  strcmp(E->d_name,"partial") == 0 ||
-	  strcmp(E->d_name,"lost+found") == 0 ||
-	  strcmp(E->d_name,".") == 0 ||
-	  strcmp(E->d_name,"..") == 0)
+      if (strcmp(E->d_name, "lock") == 0 ||
+	  strcmp(E->d_name, "partial") == 0 ||
+	  strcmp(E->d_name, "auxfiles") == 0 ||
+	  strcmp(E->d_name, "lost+found") == 0 ||
+	  strcmp(E->d_name, ".") == 0 ||
+	  strcmp(E->d_name, "..") == 0)
 	 continue;
 
       // Look in the get list and if not found nuke
@@ -844,12 +875,25 @@ pkgAcquire::UriIterator pkgAcquire::UriEnd()
 }
 									/*}}}*/
 // Acquire::MethodConfig::MethodConfig - Constructor			/*{{{*/
-// ---------------------------------------------------------------------
-/* */
-pkgAcquire::MethodConfig::MethodConfig() : d(NULL), Next(0), SingleInstance(false),
-   Pipeline(false), SendConfig(false), LocalOnly(false), NeedsCleanup(false),
-   Removable(false)
+class pkgAcquire::MethodConfig::Private
 {
+   public:
+   bool AuxRequests = false;
+};
+pkgAcquire::MethodConfig::MethodConfig() : d(new Private()), Next(0), SingleInstance(false),
+					   Pipeline(false), SendConfig(false), LocalOnly(false), NeedsCleanup(false),
+					   Removable(false)
+{
+}
+									/*}}}*/
+bool pkgAcquire::MethodConfig::GetAuxRequests() const /*{{{*/
+{
+   return d->AuxRequests;
+}
+									/*}}}*/
+void pkgAcquire::MethodConfig::SetAuxRequests(bool const value) /*{{{*/
+{
+   d->AuxRequests = value;
 }
 									/*}}}*/
 // Queue::Queue - Constructor						/*{{{*/
@@ -1074,7 +1118,7 @@ bool pkgAcquire::Queue::Cycle()
    // Look for a queable item
    QItem *I = Items;
    int ActivePriority = 0;
-   while (PipeDepth < (signed)MaxPipeDepth)
+   while (PipeDepth < static_cast<decltype(PipeDepth)>(MaxPipeDepth))
    {
       for (; I != 0; I = I->Next) {
 	 if (I->Owner->Status == pkgAcquire::Item::StatFetching)
@@ -1285,7 +1329,7 @@ bool pkgAcquireStatus::Pulse(pkgAcquire *Owner)
       else
 	 CurrentCPS = ((CurrentBytes - ResumeSize) - LastBytes)/Delta;
       LastBytes = CurrentBytes - ResumeSize;
-      ElapsedTime = (unsigned long long)Delta;
+      ElapsedTime = std::llround(Delta);
       Time = NewTime;
    }
 
@@ -1295,8 +1339,8 @@ bool pkgAcquireStatus::Pulse(pkgAcquire *Owner)
       Percent = 0;
    else
       // use both files and bytes because bytes can be unreliable
-      Percent = (0.8 * (CurrentBytes/float(TotalBytes)*100.0) +
-                 0.2 * (CurrentItems/float(TotalItems)*100.0));
+      Percent = (0.8 * (CurrentBytes/double(TotalBytes)*100.0) +
+                 0.2 * (CurrentItems/double(TotalItems)*100.0));
 
    // debug
    if (_config->FindB("Debug::acquire::progress", false) == true)
@@ -1380,7 +1424,7 @@ void pkgAcquireStatus::Stop()
    else
       CurrentCPS = FetchedBytes/Delta;
    LastBytes = CurrentBytes;
-   ElapsedTime = (unsigned long long)Delta;
+   ElapsedTime = std::llround(Delta);
 }
 									/*}}}*/
 // AcquireStatus::Fetched - Called when a byte set has been fetched	/*{{{*/
@@ -1430,6 +1474,6 @@ pkgAcquire::UriIterator::UriIterator(pkgAcquire::Queue *Q) : d(NULL), CurQ(Q), C
    }
 }
 
-APT_PURE pkgAcquire::UriIterator::~UriIterator() {}
-APT_PURE pkgAcquire::MethodConfig::~MethodConfig() {}
-APT_PURE pkgAcquireStatus::~pkgAcquireStatus() {}
+pkgAcquire::UriIterator::~UriIterator() {}
+pkgAcquire::MethodConfig::~MethodConfig() {}
+pkgAcquireStatus::~pkgAcquireStatus() {}
