@@ -315,7 +315,7 @@ bool pkgCdrom::DropRepeats(vector<string> &List,const char *Name)
       for (std::vector<APT::Configuration::Compressor>::const_iterator c = compressor.begin();
 	   c != compressor.end(); ++c)
       {
-	 std::string filename = std::string(List[I]).append(Name).append(c->Extension);
+	 std::string const filename = List[I] + Name + c->Extension;
          if (stat(filename.c_str(), &Buf) != 0)
 	    continue;
 	 Inodes[I] = Buf.st_ino;
@@ -830,7 +830,7 @@ bool pkgCdrom::Add(pkgCdromStatus *log)					/*{{{*/
    if (creation_fail == true)
    {
       UnmountCDROM(CDROM, NULL);
-      return _error->Errno("cdrom", _("List directory %spartial is missing."), listDir.c_str());
+      return _error->Errno("cdrom", _("List directory %s is missing."), (listDir + "partial").c_str());
    }
 
    // take care of the signatures and copy them if they are ok
